@@ -1,6 +1,6 @@
 // Copyright Contributors to the Open Shading Language project.
 // SPDX-License-Identifier: BSD-3-Clause
-// https://github.com/AcademySoftwareFoundation/OpenShadingLanguage
+// https://github.com/imageworks/OpenShadingLanguage
 
 
 #include <cerrno>
@@ -81,9 +81,6 @@ OSLCompiler::output_filename() const
 namespace pvt {  // OSL::pvt
 
 
-OSLCompilerImpl* oslcompiler = nullptr;
-static std::mutex oslcompiler_mutex;
-
 static ustring op_for("for");
 static ustring op_while("while");
 static ustring op_dowhile("dowhile");
@@ -443,9 +440,6 @@ OSLCompilerImpl::compile(string_view filename,
     if (m_preprocess_only && !m_generate_deps) {
         std::cout << preprocess_result;
     } else {
-        // Thread safety with the lexer/parser
-        std::lock_guard<std::mutex> lock(oslcompiler_mutex);
-        oslcompiler   = this;
         bool parseerr = osl_parse_buffer(preprocess_result);
         if (!parseerr) {
             if (shader())
@@ -490,16 +484,7 @@ OSLCompilerImpl::compile(string_view filename,
             write_oso_file(OIIO::Strutil::join(options, " "),
                            preprocess_result);
             OSL_DASSERT(m_osofile == nullptr);
-
-            oso_output.close();
-            if (!oso_output.good()) {
-                errorf(ustring(), 0, "Failed to write to \"%s\"",
-                       m_output_filename);
-                return false;
-            }
         }
-
-        oslcompiler = nullptr;
     }
 
     return !error_encountered();
@@ -540,9 +525,6 @@ OSLCompilerImpl::compile_buffer(string_view sourcecode, std::string& osobuffer,
     if (m_preprocess_only) {
         std::cout << preprocess_result;
     } else {
-        // Thread safety with the lexer/parser
-        std::lock_guard<std::mutex> lock(oslcompiler_mutex);
-        oslcompiler   = this;
         bool parseerr = osl_parse_buffer(preprocess_result);
         if (!parseerr) {
             if (shader())
@@ -581,8 +563,6 @@ OSLCompilerImpl::compile_buffer(string_view sourcecode, std::string& osobuffer,
             osobuffer = oso_output.str();
             OSL_DASSERT(m_osofile == nullptr);
         }
-
-        oslcompiler = nullptr;
     }
 
     return !error_encountered();
@@ -707,18 +687,19 @@ OSLCompilerImpl::write_oso_const_value(const ConstantSymbol* sym) const
     int nelements     = std::max(1, type.arraylen);
     if (elemtype == TypeDesc::STRING)
         for (int i = 0; i < nelements; ++i)
-            osof("\"%s\"%s", OIIO::Strutil::escape_chars(sym->strval(i)),
+            osof("\"%s\"%s", OIIO::Strutil::escape_chars(sym->get_string(i)),
                  nelements > 1 ? " " : "");
     else if (elemtype == TypeDesc::INT)
         for (int i = 0; i < nelements; ++i)
-            osof("%d%s", sym->intval(i), nelements > 1 ? " " : "");
+            osof("%d%s", sym->get_int(i), nelements > 1 ? " " : "");
     else if (elemtype == TypeDesc::FLOAT)
         for (int i = 0; i < nelements; ++i)
             osof("%.9g%s", sym->floatval(i), nelements > 1 ? " " : "");
     else if (equivalent(elemtype, TypeDesc::TypeVector))
-        for (int i = 0; i < nelements; ++i)
-            osof("%.9g %.9g %.9g%s", sym->vecval(i).x, sym->vecval(i).y,
-                 sym->vecval(i).z, nelements > 1 ? " " : "");
+        for (int i = 0; i < nelements; ++i) {
+            Vec3 v = sym->get_vec3(i);
+            osof("%.9g %.9g %.9g%s", v.x, v.y, v.z, nelements > 1 ? " " : "");
+        }
     else {
         OSL_ASSERT(0 && "Don't know how to output this constant type");
     }
